from functools import reduce, partial

def pluck(d, *ks):
    '''
    Pluck specified values from a dictionary.

    This can make for concise code together with unpacking syntax.

    Example
    -------

    >>> d = {'x': 1, 'y': 2}
    >>> pluck(d, 'x')
    [1]
    >>> pluck(d, 'y', 'x')
    [2, 1]
    >>> x, y = pluck(d, 'x', 'y')
    >>> x
    1
    >>> y
    2
    '''
    return [d[k] for k in ks]

def comp(*fns):
    '''
    Returns a function that is the passed functions composed together. Functions
    are applied from right to left.

    comp(f, g)(x) == f(g(x))

    Example
    -------

    >>> def double(n):
    ...     print(f'Doubling {n}...')
    ...     return n * 2
    >>> def inc(n):
    ...     print(f'Incrementing {n}...')
    ...     return n + 1
    >>> add1 = lambda n: n + 1
    >>> double_plus_one = comp(double, inc)
    >>> double_plus_one(3)
    Incrementing 3...
    Doubling 4...
    8
    '''
    return partial(reduce, lambda x, f: f(x), reversed(fns))



'''
A module that provides all the common data science imports.

For example:

>>> from zgulde.ds_imports import *

and all the common data science imports will be in your namespace.

Intended for quick iteration / experimentation, and explicit imports should be
used in production code.
'''

import sys
import os
import itertools as it
from functools import reduce, partial

import pandas as pd
import numpy as np

import matplotlib as mpl
import matplotlib.pyplot as plt
from matplotlib import cm
import seaborn as sns

from scipy import stats
import statsmodels.api as sm

from pydataset import data

from zgulde import comp, pluck

# check if we're running interactively
if hasattr(sys, 'ps1'):
    # turn on interactive mode in matplotlib
    plt.ion()



import pandas as pd
from pydataset import data

# better interaction w/ sklearn.metrics.confusion_matrix, e.g. auto labelling
def better_confusion_matrix(actual, predicted, labels=None):
    cm = pd.crosstab(actual, predicted)
    if labels is not None:
        cm.index = pd.Index(labels, name='Actual')
        cm.columns = pd.Index(labels, name='Predictions')
    return cm


def labelled_confusion_matrix():
    matrix = [['TP',                         'FN (Type II Error)', 'Recall = TP / (TP + FN)'],
              ['FP (Type I Error)',          'TN',                 ''],
              ['Precision = TP / (TP + FP)', '',                   '']]
    return pd.DataFrame(matrix,
                        columns=['Predicted +', 'Predicted -', ''],
                        index=['Actual +', 'Actual -', ''])

def datasets():
    datasets = data()
    shapes = datasets.apply(lambda row: data(row.dataset_id).shape, axis=1)\
                     .apply(pd.Series)\
                     .rename({0: 'nrows', 1: 'ncols'}, axis=1)
    return pd.concat([datasets, shapes], axis=1)



import pickle
import os.path
from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
from typing import List

# If modifying these scopes, delete the file token.pickle.
SCOPES = ['https://www.googleapis.com/auth/spreadsheets.readonly']

class GoogleSheetReader:
    '''A class to abstract over reading google sheet(s)

    Based on https://developers.google.com/sheets/api/quickstart/python

    1. Enable the google sheets api and obtain a json file with your credentials
    2. Run the app and sign in to your google account to allow access to your
       google sheets
    '''

    MISSING_CREDENTIALS_ERROR = '''
    Could not find file credentials file {}. You can specify a different path
    when creating an instance of this class.

    This file should contain information obtained from enabling the google
    sheets api in the google developers console https://console.developers.google.com/
    '''


    def __init__(self, credentials_path='./credentials.json',
            token_path='./token.pickle'):
        self._creds = None
        if os.path.exists(token_path):
            with open(token_path, 'rb') as token:
                self._creds = pickle.load(token)
        # If there are no (valid) credentials available, let the user log in.
        if not self._creds or not self._creds.valid:
            if self._creds and self._creds.expired and self._creds.refresh_token:
                self._creds.refresh(Request())
            else:
                if not os.path.exists(credentials_path):
                    raise FileNotFoundError(
                        self.MISSING_CREDENTIALS_ERROR.format(credentials_path))
                flow = InstalledAppFlow.from_client_secrets_file(
                    credentials_path, SCOPES)
                self._creds = flow.run_local_server()
            # Save the credentials for the next run
            with open(token_path, 'wb') as token:
                pickle.dump(self._creds, token)
        self._service = build('sheets', 'v4', credentials=self._creds)

    def read_sheet(self, sheet_id: str, sheet_range: str) -> List[List[str]]:
        '''
        Returns the given range from the given spreadsheet as a list of lists
        '''
        sheet = self._service.spreadsheets()
        result = sheet.values().get(
            spreadsheetId=sheet_id, range=sheet_range).execute()
        return result.get('values', [])
